*! version 0.4 beta August 16, 2018   Richard Williams, rwilliam@nd.edu
* Adapted from oglm.ado 2.1.2 and logit.ado Stata 12
* 0.4 -- Help file updated
* 0.3 -- default vce changed to robust. You can specify vce(oim) for binary DVs if you want


program fracglm, eclass byable(onecall) sortpreserve properties(swml svyr svyb svyj or irr rrr hr eform mi )
	version 11.2

// Replay results if that is all that is requested. 

	if replay() {
		if !inlist("`e(cmd)'", "fracglm") {
			display as error "fracglm was not the last estimation command"
			exit 301
		}
		if _by() {
			display as error "You cannot use the by command " ///
				"when replaying fracglm results"
			exit 190
		}
		else    Replay `0'
		exit
	}
	if _by() {
		local BY `"by `_byvars'`_byrc0':"'
	}
	`BY' _vce_parserun fracglm, mark(OFFset CLuster) : `0'
	if "`s(exit)'" != "" {
		ereturn local cmdline `"fracglm `0'"'
		exit
	}

	`BY' Estimate `0'
	ereturn local cmdline `"fracglm `0'"'



end

************************
program Estimate, eclass byable(recall)
	version 11.2
	syntax [varlist(default=none fv)] [if] [in]		/// Standard Stata
		[pweight fweight iweight]  [, 			///
		FROM(string)					///
		noCONstant					/// 
		OFFset(varname numeric)				///
		ASIS						///
		TECHnique(passthru)				///
		VCE(passthru)					///
		LTOLerance(passthru)				///
		TOLerance(passthru)				///
		noWARNing					///
		Robust CLuster(passthru)			/// old options
		CRITtype(passthru)				///
		SCORE(passthru)					///
		Level(cilevel)					/// display options
		DOOPT						/// NOT DOCUMENTED
		notable						/// -Replay- options
		noHeader					///
		NOCOEF						///
		Constraints(string) 				///
		or rrr irr hr EForm LOG  			///
		LRForce STOre(name)				/// special fracglm options
		LInk(string)					/// Link fncs, e.g. logit, probit
		HETero(varlist fv) SCALE(varlist fv) eq2(varlist fv)	/// Synonyms, use one only
		flip                                            /// Reverse location & scale equations; not currently supported
		HC LS						/// Affects equation labeling
		COLlinear					/// Don't check for collinearity. Affects both equations
		WALDtest(passthru)				///
		*   						/// -ml_options
		]

//	Disable flip and lrforce for now
	local flip
	*local lrforce

	
// Syntax checks, special fracglm options

	// vce(robust) is the default. For binary DVs, you may want to
	// specify vce(oim)
	if "`vce'" == "" local vce vce(robust)

	// nolog is the default unless log is specified
	if "`log'"=="" {
		local nolog "nolog"
		local qui "quietly"
	}
	
	// make sure only one eform specified.  If so, it will
	// be local macro eform
	local eform `or' `irr' `rrr' `hr' `eform'
	if `:list sizeof eform' > 1 {
		opts_exclusive "`eform'"
	}

	// Only one of hc or ls may be specified.  If so, it will
	// be local macro eqlabel
	local eqlabel `hc' `ls'
	if `:list sizeof eqlabel' > 1 {
		opts_exclusive "`eqlabel'"
	}


	if "`nocoef'" != "" {
		local table notable
		local header noheader
	}

	if `:length local doopt' {
		opts_exclusive "doopt `robust'"
		opts_exclusive "doopt `cluster'"
		opts_exclusive "doopt `score'"
		opts_exclusive "doopt `technique'"
		if `:length local ltolerance' == 0 {
			local ltolerance ltol(0)
		}
		if `:length local tolerance' == 0 {
			local tolerance tol(1e-4)
		}
		local doopt doopt halfsteponly
	}

	local vceopt =	`:length local vce'		|	///
	   		`:length local weight'		|	///
	   		`:length local cluster'		|	///
	   		`:length local robust'
	if `vceopt' {
		_vce_parse, argopt(CLuster) opt(OIM OPG Robust) old	///
			: [`weight'`exp'], `vce' `robust' `cluster'
		local vce
		if "`r(cluster)'" != "" {
			local clustvar `r(cluster)'
			local vce vce(cluster `r(cluster)')
		}
		else if "`r(robust)'" != "" {
			local vce vce(robust)
		}
		else if "`r(vce)'" != "" {
			local vce vce(`r(vce)')
		}
		if !inlist(`"`vce'"', "", "vce(oim)") {
			opts_exclusive "`doopt' `vce'"
		}
	}

	// display and maximization options
	_get_diopts diopts options, `options'
	local diopts `diopts' `eform' level(`level')
	mlopts mlopts, `options' `technique' `vce' `tolerance' `ltolerance'

	// check syntax
	if "`weight'" != "" {
		local wgt "[`weight'`exp']"
	}
	if "`offset'" != "" {
		local offopt "offset(`offset')"
	}

	// mark the estimation sample
	marksample touse
	if `:length local offset' {
		markout `touse' `offset'
	}
	if `:length local clustvar' {
		markout `touse' `clustvar', strok
	}

	tempname rules mns
	mat `rules' = J(1,4,0)


// Syntax checks, normal Stata options


	// Stata commands are inconsistent in accepting colons as an alternative to /.
	// Comma as an alternative to space sometimes works, sometimes does not.
	// We therefore change colons to / and commas to spaces, which always work.
	local constraints: subinstr local constraints ":" "/", all
	local constraints: subinstr local constraints "," " ", all

// Markout other missing values from the estimation sample
// Make sure we have cases!
	quietly count if `touse' !=0
	if r(N) == 0 {
		display as error "There are no observations left!"
		display as error "Double-check your sample selection."
		exit 2000
	}

// Additional checks on the varlist specfications

	// Get Y and X variables
	gettoken y x: varlist
	// get rid of collinear explanatory variables
	_rmcoll `x' if `touse', expand `collinear' `constant'
	local x `"`r(varlist)'"'
	_fv_check_depvar `y'

	
// Check for out of range values on dependent variable
	quietly sum `y'
	if `=r(min)' < 0 | `=r(max)' > 1 {
		display as error "One or more values of the dependent variable are " ///
		"less than 0 or greater than 1"
		exit 498
	}
	
// Link functions

	// logit is the default link if none has been specified. This
	// could be changed if someone preferred a different default. 
	if "`link'"=="" local link "logit"

	// logit link
	if "`link'"=="logit" | "`link'"=="l" {
		local link "logit"
		local link_title "Fractional Logistic Regression"
	}
	// probit is also supported
	else if "`link'"=="probit" | "`link'"=="p" | {
		local link "probit"
		local link_title "Fractional Probit Regression"
	}
	// cloglog - SPSS calls this nloglog
	else if "`link'"=="cloglog" | "`link'"=="c" |  {
		local link "cloglog"
		local link_title "Fractional Cloglog Regression"
	}
	// loglog - SPSS calls this cloglog
	else if "`link'"=="loglog" | "`link'"=="ll"  {
		local link "loglog"
		local link_title "Fractional Loglog Regression"
	}
	// cauchit
	else if "`link'"=="cauchit" | "`link'"=="ca" | "`link'"=="cau"  {
		local link "cauchit"
		local link_title "Fractional Cauchit Regression"
	}
	// log - This link is experimental, may not work right
	else if "`link'"=="log"   {
		local link "log"
		local link_title "Fractional Log Regression"
	}
	// non-supported link
	else {
		display ""
		display as error ///
			"{yellow}`link'{red} is not a legal link function"
		display ""
		exit 198
	}



// heteroskedasticity/ scale/ eq2 option

	// Can only specify one of hetero, scale, eq2
	local het_options = 0
	foreach opt in hetero scale eq2 {
		if "``opt''"!="" local het_options = `het_options' + 1
	}
	if `het_options' > 1 {
		display as error "hetero, scale & eq2 are synonyms - use only one of them"
		exit 198
	}
	
	local hetero `scale' `hetero' `eq2'

	if "`hetero'" !="" {
		_rmcoll `hetero' if `touse', expand `collinear' `constant'
		local hetero `r(varlist)'
	}

// Flip the 2 equations if so requested.  This is mainly useful
// if using the sw or nestreg prefixes. Not currently supported though.
	if "`flip'"!="" {
		local vhetero `x'
		local vx `hetero'
		local hetero `vhetero'
		local x `vx'
	}

	local Numx: word count `x'
	local Numh: word count `hetero'

	// labeling of equations & model
	if `Numh' {
		local link_title Heteroskedastic `link_title'
		markout `touse' `hetero'
	}
	if "`eqlabel'"=="" {
		local eq1label `y'
		local eq2label lnsigma
	}
	else if "`eqlabel'"=="hc" {
		local eq1label choice
		local eq2label variance
	}
	else if "`eqlabel'"=="ls" {
		local eq1label location
		local eq2label scale
	}


// Build equations for full model to be estimated. There are 2 possible
// types of models, depending on whether or not hetero is specified.
	if `Numh' {
		local eqs (`eq1label': `y'=`x', `constant' `offopt' `expopt') (`eq2label': `hetero', noconstant)
		local eval frachet`link'_lf()
	}
	else {
		local eqs (`eq1label': `y'=`x', `constant' `offopt' `expopt')
		local eval frac`link'_lf()
	}

// Start values -- Do constant-only model for both het & nonhet models. 
// This is what SPSS apparently does
	local starteval frac`link'_lf()
	local eqsx (`eq1label': `y'=, `constant' `offopt' `expopt')	
	if "`constant'" != "noconstant" {
		`qui' display as txt _n "Fitting constant-only model:"
		ml model lf `starteval' `eqsx' `wgt' if `touse', 		///
			constraints(`constraints') 		 	 	///
			waldtest(-1) title(`link_title') 			///
			collinear missing maximize nocnsnotes 			///
			`score' `nolog' `mlopts'  				///
			`doopt' `crittype'
		tempname myh
		mat `myh' = e(b)
		local continue "init(`myh')"
  
	}
	if "`e(ll)'"!="" local LL0 = `e(ll)'



*****************************************

// Estimate the final model; and add some stats
	`qui' display as txt _n "Fitting full model:"
	ml model lf `eval' `eqs' `wgt' if `touse', 		///
		constraints(`constraints') 	 	 	///
		`waldtest' `continue' title(`link_title') 	///
		collinear missing maximize nocnsnotes 		///
		`score' `nolog' `mlopts'  			///
		`doopt' `crittype'

	if "`LL0'"!="" ereturn scalar ll_0 = `LL0'

	// Compute McFadden's Pseudo R^2 if necessary info exists, i.e. it won't 
	// be possible after using svy because svy does not return it.
	// Also, Stata ml does not rescale pweights, but fracglm will
	if (e(ll) < . & e(ll_0) < .) {
		if "`weight'" == "pweight" {
			local wgtvar: word 2 of `=e(wexp)'
			quietly sum `wgtvar' if e(sample)
			ereturn scalar ll = e(ll)/ r(mean)
		}
		if "`constant'" != "noconstant" ereturn scalar r2_p = 1 - (e(ll) / e(ll_0) )
	}

	// Stata reports a Wald rather than LR test for the model.
	// The lrforce parameter makes Stata report a LR test instead, provided
	// crittype is log likelihood.  Use this at your own risk
	local crittype = e(crittype)
	local chi2type = e(chi2type)
	if "`lrforce'"!="" & "`crittype'"=="log likelihood" & /// 
		"`chi2type'"!="LR" & "`constant'" != "noconstant"  {
		ereturn local chi2type "LR"
		ereturn scalar chi2 = -2 * (e(ll_0) - e(ll))
		ereturn scalar p = chi2tail(e(df_m),e(chi2))
	}



// Return assorted values
	ereturn local xvars `x'
	ereturn local hetero `hetero'
	ereturn local link `link'
	ereturn matrix rules `rules'

// Final cleanup

	macro drop Link

	ereturn local predict "fracglm_p"
	ereturn local cmd "fracglm"

	ereturn local marginsok pr xb
	ereturn local marginsnotok DEFAULT sigma stdp

// display the results

	Replay, `diopts' store(`store')
	
end


************************

program Replay, eclass
	version 11.2
	syntax [,				///
	Level(cilevel)			 	///
	or irr rrr hr EForm 			///
	STOre(name)				///
	* ]
	
	// make sure only one eform specified.  If so, it will
	// be local macro eform
	local eform `or' `irr' `rrr' `hr' `eform'
	if `:list sizeof eform' > 1 {
		opts_exclusive "`eform'"
	}

	
	// Make sure equation names show if hetero equation is being used
	if "`e(hetero)'" !="" local showeqns showeqns
	
	_get_diopts diopts , `options' 
	local diopts `diopts' `eform' level(`level') `showeqns'
	ml display , `diopts'
	
	
	// Store results if requested
	if "`store'"!="" estimates store `store'

end

