version 11.2
clear all
* Mata evaluator routines for fracglm
* Adapted from oglm_ll.ado 2.1.2
* & pp. 193-194 of Maximum Likelihood Estimation
* with Stata, 4th edition, Gould, Pittblado & Poi

// ----------------------------------------------------

mata:
// 
// Probit and HetProbit
void fracprobit_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
lnfj = depvar:*ln(normal(xb)) + (1:-depvar):*ln(normal(-xb))
}

void frachetprobit_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj )
{
real vector depvar, xb, z
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
z = moptimize_util_xb(ML, b, 2)
lnfj = depvar:*ln(normal(xb:/exp(z))) + (1:-depvar):*ln(normal(-xb:/exp(z)))
}

// ----------------------------------------------------
// Logit and HetLogit
void fraclogit_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
lnfj = depvar:*ln(invlogit(xb)) + (1:-depvar):*ln(invlogit(-xb))
}

void frachetlogit_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb, z
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
z = moptimize_util_xb(ML, b, 2)
lnfj = depvar:*ln(invlogit(xb :/exp(z))) + (1:-depvar):*ln(invlogit(-xb:/exp(z)))
}

// ----------------------------------------------------
// Cloglog and HetCloglog
void fraccloglog_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
lnfj = depvar:*ln(invcloglog(xb)) + (1:-depvar):*ln(1:-invcloglog(xb))
}

void frachetcloglog_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb, z
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
z = moptimize_util_xb(ML, b, 2)
lnfj = depvar:*ln(invcloglog(xb:/exp(z))) + (1:-depvar):*ln(1:-invcloglog(xb:/exp(z)))
}

// ----------------------------------------------------
// Loglog and HetLoglog
void fracloglog_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
lnfj = (1:-depvar):*ln(invcloglog(-xb)) + depvar:*ln(1:-invcloglog(-xb))
}

void frachetloglog_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb, z
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
z = moptimize_util_xb(ML, b, 2)
lnfj = (1:-depvar):*ln(invcloglog(-xb:/exp(z))) + depvar:*ln(1:-invcloglog(-xb:/exp(z)))
}

// ----------------------------------------------------
// Cauchit and HetCauchit
void fraccauchit_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
lnfj = depvar:*ln(.5:-atan(-xb):/pi())+(1:-depvar):*ln(.5:+atan(-xb):/pi())
}

void frachetcauchit_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb, z
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
z = moptimize_util_xb(ML, b, 2)
lnfj = depvar:*ln(.5:-atan(-xb:/exp(z)):/pi())+(1:-depvar):*ln(.5:+atan(-xb:/exp(z)):/pi())
}

// ----------------------------------------------------
// Log and HetLog - This is the most questionable link
void fraclog_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
lnfj = depvar:*ln(exp(xb)) + (1:-depvar):*ln(1:-exp(xb))
}

void frachetlog_lf(transmorphic scalar ML, real rowvector b, real colvector lnfj)
{
real vector depvar, xb, z
depvar = moptimize_util_depvar(ML, 1)
xb = moptimize_util_xb(ML, b, 1)
z = moptimize_util_xb(ML, b, 2)
lnfj = depvar:*ln(exp(xb:/exp(z))) + (1:-depvar):*ln(1:-exp(xb:/exp(z)))
}

// ----------------------------------------------------

end

