{smcl}
{* Last revised August 6, 2018}{...}
{hline}
help for {hi:fracglm} beta version 0.4
{hline}

{title:Fractional Response Generalized Linear Models}

{p 8 15 2}
	{cmdab:fracglm} {it:depvar} [{it:indepvars}]
	[{it:weight}] [{cmd:if} {it:exp}] [{cmd:in} {it:range}]
	[{cmd:,} 
	{cmd:link(}{it:logit/probit/cloglog/loglog/cauchit/log}{cmd:)}
	{cmdab:sto:re(}{it:name}{cmd:)} 
	{cmd:vce(}{it:vcetype}{cmd:)}
	{cmdab:c:onstraints:(}{it:clist}{cmd:)}
	{cmdab:l:evel:(}{it:#}{cmd:)} 
	{opt or} {opt irr} {opt rrr} {opt eform} {opt hr}
	{cmdab:log}
	{cmdab:het:ero(}{it:varlist}{cmd:)} {opt scale(varlist)} {opt eq2(varlist)}
	{opt hc} {opt ls}
	{it:maximize_options} ]

{p 4 4 2}
{cmd:fracglm} supports factor variables and the {cmd:margins} command and requires Stata 
version 11.2 or later.

{p 4 4 2}
{cmd:fracglm} shares the features of all estimation commands; see help
{help est}.    
{cmd:fracglm} typed without arguments redisplays previous results.  
The following options may be given when redisplaying results:

{p 8 8 2}
	{cmdab:sto:re}
	{cmd:or} {opt irr} {opt rrr} {opt hr} {opt eform} 
	{cmdab:l:evel:(}{it:#}{cmd:)}

{p 4 4 2}
{opt by}, {opt svy}, {opt nestreg}, {opt stepwise}, {opt xi} and possibly
other prefix commands are allowed; see help {help prefix}. 

{p 4 4 2} {cmd:fweight}s, {cmd:iweight}s, and {cmd:pweight}s are
allowed; see help {help weights}.

{title:Syntax for predict}

{p 8 16 2}
{cmd:predict}
{dtype}
{it:{help newvar:newvar}}
{ifin}
[{cmd:,} {it:statistic}
]

{synoptset 11 tabbed}{...}
{synopthdr :statistic}
{synoptline}
{syntab :Main}
{synopt :{opt p:r}}predicted probabilities; the default{p_end}
{synopt :{opt xb}}linear prediction{p_end}
{synopt :{opt s:igma}}the standard deviation{p_end}
{synopt :{opt stdp}}standard error of the linear prediction{p_end}
{synopt :{opt score}}calculates the equation-level score, the derivative of 
the log likelihood with respect to the linear prediction.{p_end}
{synoptline}
{p2colreset}{...}

{p 4 4 2}
These statistics are available both in and out of sample; type
"{cmd:predict} {it:...} {cmd:if e(sample)} {it:...}" if wanted only for the
estimation sample.


{title:Description}

{p 4 4 2}{cmd:fracglm} estimates Fractional Response Generalized Linear Models 
(e.g. Fractional Probit, Fractional Logit) with or
without heteroskedasticity. Fractional response variables range in value between
0 and 1.  An example of a fractional response variable would be the percentage of
employees covered by an employer's pension plan.
{cmd:fracglm} also works with binary 0/1 dependent variables.
{cmd:fracglm} supports multiple link functions,
including logit (the default), probit, complementary log-log, log-log, log
and cauchit. When
these models include equations for heteroskedasticity they are also
known as heterogeneous choice/ location-scale / heteroskedastic
regression models. 

{p 4 4 2}{cmd:fracglm} fills gaps left by other Stata commands.
Commands like {cmd:logit} and {cmd:probit} do not
allow for fractional response variables. {cmd:glm} can estimate some fractional
response models but does not allow an equation for heteroskedasticity. In Stata 14,
{cmd:fracreg} will do many of the same things that {cmd:fracglm} does. But even so,
{cmd:fracreg} does not support as many link functions and requires Stata 14 or higher.
{cmd:fracglm} only requires Stata 11.2.

{p 4 4 2}Several special cases of  generalized linear models can also be
estimated by {cmd:fracglm}, including the binomial generalized linear models of
{cmd:logit}, {cmd:probit} and {cmd:cloglog} (which also assume
homoskedasticity), {cmd:hetprob}, as well as similar models that are not otherwise
estimated by Stata.  This makes {cmd:fracglm} particularly useful for
testing whether constraints on a model (e.g. homoskedastic errors) are
justified, or for determining whether one link function is more
appropriate for the data than are others.

{p 4 4 2}Other features of {cmd:fracglm} include support for linear constraints, making
it possible, for example, to impose and test the constraint that the
effects of x1 and x2 are equal.  {cmd:fracglm} works with several prefix
commands, including {cmd:by}, {cmd:nestreg}, {cmd:xi}, {cmd:svy} and {cmd:sw},
as well as some post-estimation commands, such as {cmd:margins}.
Its {cmd:predict} command includes the ability to compute estimated
probabilities.  {cmd:fracglm} was
inspired by the SPSS PLUM routine but differs somewhat in its
terminology and labeling of links.

{p 4 4 2}By default, {cmd:fracglm} uses robust standard errors. If your dependent
variable is binary, you may wish to specify {opth vce(oim)} instead.

{p 4 4 2} WARNINGS: (1) The use of stepwise regression is discouraged by many
statisticians, and it can be especially problematic with heteroskedastic models.
For example, the default Wald tests conducted by the {opt nestreg}
and {opt sw} prefix commands can give incorrect results when the
same variable appears in both the location and scale equations.  In
such cases it is recommended that you use {opt nestreg}'s and {opt sw}'s
likelihood ratio test options. (2) The dependent variable must range in value
between 0 and 1. Values outside that range will result in a fatal error.
(3) heteroskedastic models can be difficult to estimate, especially if the 
dependent variable is coded 0/1 (rather than fractional) and when there is
substantial overlap between the variables included in the two equations.
The {opt difficult} option may help in some cases.
(4) {cmd:fracglm} is not appropriate for ordinal dependent variables.
In such cases consider using the user-written {cmd:oglm} routine, available from SSC.


{title:Options}

{p 4 8 2} {opt link(logit/probit/cloglog/loglog/cauchit/log)} specifies
the link function to be used.  The legal values are {opt link(logit)},
{opt link(probit)}, {opt link(cloglog)}, {opt link(loglog)},  {opt link(log)} 
and {opt link(cauchit)} which
can be abbreviated as {opt link(l)}, {opt link(p)}, {opt link(c)}, {opt link(ll)},
{opt link(ca)} and {opt link(log)}. {opt link(logit)} is the default if the option is
omitted.


{p 8 8 2} The following advice is adapted from Norusis (2005, p. 84):
Probit and logit models are reasonable choices when the changes in the
cumulative probabilities are gradual. If there are abrupt changes,
other link functions should be used. The log-log link may be a good
model when the cumulative probabilities increase from 0 fairly slowly
and then rapidly approach 1. If the opposite is true, namely that the
cumulative probability for lower scores is high and the approach to 1
is slow, the complementary log-log link may describe the data.

{p 8 8 2} WARNING: Programs differ in the names used for some
links.  Stata's loglog link corresponds to SPSS PLUM's cloglog link;
and Stata's cloglog link is called nloglog in SPSS.

{p 4 8 2} {opt hetero(varlist)}, {opt scale(varlist)} and {opt eq2(varlist)}
are synonyms (use only one of them) and can be used to specify the
variables believed to affect heteroskedasticity in heterogeneous
choice/ location-scale models.

{p 4 8 2} {opt hc} and {opt ls} affect how the equations are labeled.
If {opt hc} is used, then, consistent with the literature on
heterogeneous choice, the equations are labeled "choice" and
"variance". If {opt ls} is used, the equations are labeled "location"
and "scale", which is consistent with SPSS PLUM and other published
literature.  If neither option is specified, then the
scale/heteroskedasticity equation is labeled "lnsigma", which is
consistent with other Stata programs such as {cmd:hetprob}.

{p 4 8 2} {cmd:store(}{it:name}{cmd:)} causes the command
{cmd:estimates store {it:name}} to be executed when {cmd:fracglm}
finishes.  This is useful for when you wish to estimate a series of
models and want to save the results. See help {help estimates}. 

{p 8 8 2} WARNING: The {opt store} option may not work correctly when
the {opt svy} prefix is used.

{p 4 8 2} {cmd:log} displays the iteration log. By default it is
suppressed.

{p 4 8 2} {cmd:or} reports the estimated coefficients transformed to
relative odds ratios, i.e., exp(b) rather than b; see {hi:[R] ologit}
for a description of this concept.  Options {cmd:rrr}, {cmd:eform},
{cmd:irr} and {opt hr} produce identical results (labeled differently)
and can also be used. It is up to the user to decide whether the
exp(b) transformation makes sense given the link function used, e.g.
it probably doesn't make sense when using the probit link.

{p 4 8 2} {cmd:constraints(}{it:clist}{cmd:)} specifies the linear
constraints to be applied during estimation.  The default is to
perform unconstrained estimation.  Constraints are defined with the 
{help constraint} command. {cmd:constraints(1)} specifies that the
model is to be constrained according to constraint 1;
{cmd:constraints(1-4)} specifies constraints 1 through 4;
{cmd:constraints(1-4,8)} specifies 1 through 4 and 8.  

{p 4 8 2} {opth vce(vcetype)} specifies the type of standard error reported, 
which includes types that are derived from asymptotic theory, that are robust
to some kinds of misspecification, that allow for intragroup correlation, 
and that use bootstrap or jackknife methods.
{it:vcetype} may be {opt oim}, {opt r:obust}, {opt cl:uster} {it:clustvar},
{opt boot:strap}, or {opt jack:knife}. The default is {opt r:obust}. For binary
dependent variables, you may wish to specify {opt oim} instead.

{p 4 8 2} {cmd:level(}{it:#}{cmd:)} specifies the confidence level in
percent for the confidence intervals of the coefficients; see help 
{help level}.

{p 4 8 2} {it:maximize_options} control the maximization process; see
help {help maximize}.  You should never have to specify most of these.
However, the {opt difficult} option can sometimes be useful with models
that are running very slowly or not converging at all.


{title:Options for predict}

{phang}
{opt pr}, the default, calculates the predicted probabilities.

{phang}
{opt xb} calculates the linear prediction.  You specify one new
variable, for example, {cmd:predict linear, xb}.

{phang}
{opt sigma} calculates the standard deviation, also known as the
scale.  You specify one new variable, for example, {cmd:predict sigma, s}. 
If the model does not include an equation for heteroskedasticity
then the predicted sigma value is missing for all cases.

{phang}
{opt stdp} calculates the standard error of the linear prediction.  You
specify one new variable, for example, {cmd:predict se, stdp}.

{phang}
{opt scores} calculates equation-level score variables.  

{title:Examples}

{p 4 4 2} {it}Example 1. Fractional models. {sf} You can estimate both non-heteroskedastic and heteroskedactic
fractional response models. With fractional models you probably want to use the default vce(robust) option.

{p 8 12 2}{cmd:.  use https://www3.nd.edu/~rwilliam/statafiles/401kpart, clear}{p_end}
{p 8 12 2}{cmd:.  fracglm prate mrate ltotemp age i.sole, vce(robust) link(p)}{p_end}
{p 8 12 2}{cmd:.  use https://www3.nd.edu/~rwilliam/statafiles/401kpart, clear}{p_end}
{p 8 12 2}{cmd:.  fracglm prate mrate ltotemp age i.sole, het(mrate ltotemp age i.sole) vce(robust) link(p)}{p_end}
{p 8 12 2}{cmd:.  test [lnsigma]}{p_end}

{p 4 4 2} {it}Example 2. Basic binary models. {sf} By default, {cmd:fracglm} will estimate the same
models as {cmd:logit} if the dependent variable is binary.  The {opt store} option is convenient for saving results if you
want to contrast different models. However, the default is vce(robust) so you may wish to change that to
vce(oim) for binary dependent variables.

{p 8 12 2}{cmd:. use http://www.indiana.edu/~jslsoc/stata/spex_data/ordwarm2.dta, clear}{p_end}
{p 8 12 2}{cmd:. fracglm  warmlt3 yr89 male white age ed prst, vce(oim)}{p_end}
{p 8 12 2}{cmd:. fracglm  warmlt3 yr89 male white age ed prst, store(m1) vce(oim)}{p_end}
{p 8 12 2}{cmd:. fracglm  warmlt3 yr89 male white age ed prst}{p_end}

{p 4 4 2} {it}Example 3. Survey data estimation.{sf}  

{p 8 12 2}{cmd:. webuse nhanes2f, clear}{p_end}
{p 8 12 2}{cmd:. svy: fracglm diabetes female black age age2}{p_end}
{p 8 12 2}{cmd:. svy, subpop(female): fracglm diabetes black age age2}{p_end}

{p 4 4 2} {it}Example 4.  The {cmd:predict} command. {sf} 

{p 8 12 2}{cmd:. use http://www.indiana.edu/~jslsoc/stata/spex_data/ordwarm2.dta, clear}{p_end}
{p 8 12 2}{cmd:. quietly fracglm  warmlt3 yr89 male white age ed prst}{p_end}
{p 8 12 2}{cmd:. predict p}{p_end}

{p 4 4 2} {it}Example 5.  Constrained logistic regression. {sf}

{p 8 12 2}{cmd:. use http://www.indiana.edu/~jslsoc/stata/spex_data/ordwarm2.dta, clear}{p_end}
{p 8 12 2}{cmd:. recode warm (1 2  = 0)(3 4 = 1), gen(agree)}{p_end}
{p 8 12 2}{cmd:. * Constrain the effects of male and white to be equal}{p_end}
{p 8 12 2}{cmd:. constraint 1 male = white}{p_end}
{p 8 12 2}{cmd:. fracglm agree yr89 male white age ed prst, store(constrained) c(1)}{p_end}
{p 8 12 2}{cmd:. fracglm agree yr89 male white age ed prst, store(unconstrained) }{p_end}
{p 8 12 2}{cmd:. lrtest constrained unconstrained}{p_end}


{p 4 4 2} {it}Example 6.  Other link functions. {sf} By default,
{cmd:fracglm} uses the logit link. If you prefer, however, you can
specify probit, complementary log log, log log or log links.  In the following example, the
same model is estimated using each of the links supported by fracglm (note that {opt link(log)}
is considered experimental and possibly wrong.}

{p 8 12 2}{cmd:. use http://www.indiana.edu/~jslsoc/stata/spex_data/ordwarm2.dta, clear}{p_end}
{p 8 12 2}{cmd:. fracglm warmlt3 yr89 male white age ed prst, link(l)}{p_end}
{p 8 12 2}{cmd:. fracglm warmlt3 yr89 male white age ed prst, link(p)}{p_end}
{p 8 12 2}{cmd:. fracglm warmlt3 yr89 male white age ed prst, link(c)}{p_end}
{p 8 12 2}{cmd:. fracglm warmlt3 yr89 male white age ed prst, link(ll)}{p_end}
{p 8 12 2}{cmd:. fracglm warmlt3 yr89 male white age ed prst, link(ca)}{p_end}


{p 4 4 2} {it}Example 7.  Prefix commands. {sf} {cmd:fracglm} supports many of
Stata's prefix commands. For example,

{p 8 12 2}{cmd:. use http://www.indiana.edu/~jslsoc/stata/spex_data/ordwarm2.dta, clear}{p_end}
{p 8 12 2}{cmd:. sw, pe(.05): fracglm warmlt3 yr89 male}{p_end}
{p 8 12 2}{cmd:. xi: fracglm warmlt3 yr89 i.male}{p_end}
{p 8 12 2}{cmd:. nestreg: fracglm warmlt3 (yr89 male  white age) (ed prst)}{p_end}

{p 4 4 2} {it}Example 8.  heteroskedasticity/scale/eq2 option. {sf}
The {opt het}, {opt scale} and {opt eq2} options are synonyms, use
whichever one you prefer.  {opt ls} and {opt hc} are optional and
affect whether the equations are labeled consistently with the
heterogeneous choice or location-scale literature. If also using the
{opt sw} or {opt nestreg} prefix commands, you should use their
likelihood ratio test options since the default Wald tests can be
wrong when the same variable appears in both the location and scale
equations. Note that it is possible to estimate a heteroskedasticity-only
model, and that the variables in the two equations do not need to
be the same.

{p 8 12 2}{cmd:. use http://www.indiana.edu/~jslsoc/stata/spex_data/ordwarm2.dta, clear}{p_end}
{p 8 12 2}{cmd:. fracglm warmlt3 yr89 male white age ed prst, het(yr89) hc}{p_end}
{p 8 12 2}{cmd:. fracglm warmlt3 yr89 male white age ed prst, scale(male white) ls link(p)}{p_end}
{p 8 12 2}{cmd:. fracglm warmlt3, eq2(male)}{p_end}
{p 8 12 2}{cmd:. sw, pe(.05) lr: fracglm warmlt3 yr89 male white age ed prst, het(male) }{p_end}
{p 8 12 2}{cmd:. nestreg, lr: fracglm warmlt3 yr89 male white age ed prst, het(white)}{p_end}

{p 4 4 2} {it}Example 9.  Factor variables and the margins command. {sf} {cmd:fracglm} works with
factor variables and the {cmd:margins} command. Keep in mind that not all prefix commands work when factor
variables are used.

{p 8 12 2}{cmd:. use http://www.indiana.edu/~jslsoc/stata/spex_data/ordwarm2.dta, clear}{p_end}
{p 8 12 2}{cmd:. recode warm (1 2  = 0)(3 4 = 1), gen(agree)}{p_end}
{p 8 12 2}{cmd:. fracglm agree i.yr89 i.male i.white age ed prst}{p_end}
{p 8 12 2}{cmd:. margins yr89 male white}{p_end}
{p 8 12 2}{cmd:. margins, dydx(*) }{p_end}

{title:Author}

{p 5 5}
Richard Williams{break}
Notre Dame Department of Sociology{break}
Richard.A.Williams.5@ND.Edu{break}
{browse "https://www.nd.edu/~rwilliam/"}{p_end}


{title:Acknowledgements}

{p 5 5}The documentation and source code for several Stata commands
(e.g. {opt hetpr_p}) were major aids in developing the {cmd:fracglm}
documentation and in adding support for the {cmd:predict} command.
Much of the code is adapted from {it}Maximum Likelihood Estimation
with Stata, Third Edition{sf}, by William Gould, Jeffrey Pitblado and
William Sribney.  SPSS's PLUM routine helped to inspire {opt fracglm} and
provided a means for double-checking the accuracy of the program.

{p 5 5} Joseph Hilbe, Mike Lacy and Rory Wolfe provided stimulating and helpful
comments. Jeff Pitblado helped me with several programming issues. 
Kerry Kammire played a major role in updating fracglm to work with factor variables
and the margins command.

{title:References}

{p 5 5}Alvarez, R. Michael and John Brehm. 1995.  "American
Ambivalence towards Abortion Policy: Development of a Heteroskedastic
Probit Model of Competing Values." American Journal of Political
Science 39(4):1055-82.

{p 5 5}Hardin, James and Joseph Hilbe.  2001. "Generalized Linear
Models and Extensions." College Station, TX: Stata Press.

{p 5 5}Long, J. Scott and Jeremy Freese.  2006.  "Regression Models
for Categorical Dependent Variables Using Stata, 2nd Edition." College
Station, Texas: Stata Press.

{p 5 5}Norusis, Marija.  2005.  "SPSS 13.0 Advanced Statistical
Procedures Companion."  Upper Saddle River, New Jersey: Prentice Hall. See
especially the chapter on SPSS PLUM, available on the web at{break}
{browse "http://www.norusis.com/pdf/ASPC_v13.pdf"}{p_end}

{title:Suggested citations if using {cmd:fracglm} in published work }

{p 5 5}{cmd:fracglm} is not an official Stata command. It is a free
contribution to the research community, like a paper. Please cite it
as such.

{p 5 5}Williams, Richard.  2009. "Using Heterogeneous Choice Models To 
Compare Logit and Probit Coefficients Across Groups" 
Sociological Methods & Research 37(4): 531-559.A pre-publication version
is available at {break} 
{browse "https://www.nd.edu/~rwilliam/oglm/RW_Hetero_Choice.pdf"}.

{p 5 5}Williams, Richard.  2010. "Fitting Heterogeneous Choice 
Models with oglm."  The Stata Journal 10(4):540-567.  
A pre-publication version is available at {break}
{browse "https://www.nd.edu/~rwilliam/oglm/oglm_Stata.pdf"}.

{p 5 5}Williams, Richard.  2006.  "Generalized Ordered Logit/ Partial
Proportional Odds Models for Ordinal Dependent Variables." The Stata 
Journal 6(1):58-82.  
The published article can be found at 
{browse "http://www.stata-journal.com/article.html?article=st0097"}.
Information on
updates to the program since the article was published are available at {break}
{browse "https://www.nd.edu/~rwilliam/gologit2/gologit2.pdf"}. 


{p 5 5} {cmd:gologit2} and {cmd:oglm} are related program and may be more appropriate
than {cmd:fracglm} for some purposes.  

{p 5 5}I would appreciate an email notification if you use  fracglm in published 
work, as well as a citation of one or more of the sources listed above.  Also 
feel free to email me if you have comments about the program or its documentation.


{title:Also see}

{p 4 13 2} Online: {help estcom}, {help postest}, 
{help constraint}, {help logit}, {help probit}, {help hetprob},
{help svy}, {help margins}, {help fvvarlist}, {help prefix},
{help gologit2} (if installed).
