*! version 0.4 beta August 16, 2018   Richard Williams, rwilliam@nd.edu
program define fracglm_p
	version 6, missing
	local c hetero `=e(hetero)'
	local link `=e(link)'
	
	syntax [anything] [if] [in] [, SCores * ]
	if `"`scores'"' != "" {
		ml score `0'
		exit
	}

		/* Step 1:
			place command-unique options in local myopts
			Note that standard options are
			LR:
				Index XB Cooksd Hat 
				REsiduals RSTAndard RSTUdent
				STDF STDP STDR CONstant(varname) 
			SE:
				Index XB STDP CONstant(varname)
		*/
	local myopts "Pr Sigma"

		/* Step 2:
			call _propts, exit if done, 
			else collect what was returned.
		*/
	_pred_se "`myopts'" `0'
	if `s(done)' { exit }
	local vtyp  `s(typ)'
	local varn `s(varn)'
	local 0 `"`s(rest)'"'


		/* Step 3:
			Parse your syntax.
		*/
	syntax [if] [in] [, `myopts' noOFFset]
	
	* Determine link used

	tempvar num denom
	if "`link'" == "probit" {
		local fnc normprob(`num')
		local fnchet normprob(`num'/`denom')
	}
	else if "`link'" == "logit" {
		local fnc invlogit(`num')
		local fnchet invlogit(`num'/`denom')
	}
	else if "`link'" == "cloglog" {
		local fnc invcloglog(`num')
		local fnchet invcloglog(`num'/`denom')
	}
	else if "`link'" == "loglog" {
		local fnc 1 - invcloglog(-(`num'))
		local fnchet 1 - invcloglog(-(`num'/`denom'))
	}
	else if "`link'" == "log" {
		local fnc exp(`num')
		local fnchet exp(`num'/`denom')
	}
	else if "`link'" == "cauchit" {
		local fnc (1 - .5 - (1/_pi) * atan(-`num'))
		local fnchet (1 - .5 - (1/_pi) * atan(-`num'/`denom'))
	}
	else {
		display as error "Unsupported link function"
		exit 198
	}

	if "`sigma'" != "" {
		if "`hetero'"== "" {
			display as error "Warning! All values are missing " ///
				"because there was no hetero/scale equation"
			qui gen `vtyp' `varn' = .
			label var `varn' "Sigma"

		}
		else {
			tempvar lnvar
			_predict double `lnvar' `if' `in', xb eq(#2) `offset'
			qui gen `vtyp' `varn' = exp(`lnvar')
			label var `varn' "Sigma"
		}
	}
	else {
		if "`pr'"=="" {
		   noi di in gr "(option pr assumed; Pr(`e(depvar)'))"
		}
	        _predict double `num' `if' `in', xb eq(#1) `offset'
		if "`e(hetero)'" !="" {
			_predict double `denom' `if' `in', xb eq(#2) `offset'
			qui replace `denom' = exp(`denom')
			qui gen `vtyp' `varn' = `fnchet'
			label var `varn' "Pr(`e(depvar)')"
		}
		else {
			qui gen `vtyp' `varn' = `fnc'
			label var `varn' "Pr(`e(depvar)')"
		}
	}
end
